//
//  VideoController.m
//  Log-Laplace
//
//  Created by Kévin Toublanc on 12/04/12.
//  Copyright (c) 2012 Polytech'Tours. All rights reserved.
//

#import "VideoController.h"
#import "AppDelegate.h"
#import "LutMatrix.h"

@interface VideoController (private)

-(void) captureSessionInitialization;

@end

@implementation VideoController (private)

-(void)captureSessionInitialization{
    /*Input Device*/
    AVCaptureDeviceInput * captureInput = [AVCaptureDeviceInput 
                                           deviceInputWithDevice:[AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo] 
                                           error:nil];
    /*Output Device*/
    AVCaptureVideoDataOutput *captureOutput = [[AVCaptureVideoDataOutput alloc] init];
    captureOutput.alwaysDiscardsLateVideoFrames = YES;
	//captureOutput.minFrameDuration = CMTimeMake(1, 10);
    
    /*We create a serial queue to handle the processing of our frames*/
	dispatch_queue_t queue;
	queue = dispatch_queue_create("cameraQueue",NULL);
	[captureOutput setSampleBufferDelegate:self queue:queue];
	dispatch_release(queue);
    
    // Set the video output to store frame in BGRA (It is supposed to be faster)
	NSString* key = (NSString*) kCVPixelBufferPixelFormatTypeKey; 
	NSNumber* value = [NSNumber numberWithUnsignedInt:kCVPixelFormatType_32BGRA]; 
	NSDictionary* videoSettings = [NSDictionary dictionaryWithObject:value forKey:key]; 
	[captureOutput setVideoSettings:videoSettings];
    
    /*We add input and output*/
	[self.captureSession addInput:captureInput];
	[self.captureSession addOutput:captureOutput];
    
	/*We add the preview layer*/
	self.previewLayer = [AVCaptureVideoPreviewLayer layerWithSession: self.captureSession];
	self.previewLayer.frame = [[[UIApplication sharedApplication] delegate] window].frame;
	self.previewLayer.videoGravity =  AVLayerVideoGravityResizeAspectFill;   
    [self.cameraView.layer addSublayer: self.previewLayer];
    
}

@end

@interface VideoController ()

@end

@implementation VideoController
@synthesize previewLayer = _previewLayer;
@synthesize captureSession = _captureSession;
@synthesize cameraView = _cameraView;
@synthesize test;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[[[UIApplication sharedApplication] delegate] window] addSubview:self.cameraView];
    
    self.view.backgroundColor = [UIColor clearColor];
    
    [[(AppDelegate*)[[UIApplication sharedApplication]delegate] window] addSubview:self.navigationController.view];
    
    [self captureSessionInitialization];
    
    [[self captureSession] startRunning];
    
    UITapGestureRecognizer * tapGest = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap:)];
    tapGest.numberOfTapsRequired = 1;
    tapGest.delaysTouchesBegan = YES;
    [self.view addGestureRecognizer:tapGest];
    
    /*Config mask*/
    mask = malloc(9 * sizeof(int));
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            if(i == 1 && j == 1){
                mask[(int)(i * 3 + j)] = 4;
            }else {
                if((i == 1 && (j==0 || j==2) )||((i==0 || i==2) && j==1))
                mask[(int)(i * 3 + j)] = -1;
            }
        }
    }
    maskSize.width = 3;
    maskSize.height = 3;
    
    /*Config LUT*/
    lut = getLutMatrix(0.33, 0.33, 0.34);
    

    
    /*Initi convo*/
    convolution = [SimpleConvolutionMask new];
    convolution.mask = mask;
    convolution.maskSize = maskSize;
    convolution.delegate = nil;
    [convolution setLUT:lut];

    
}

-(void)viewDidAppear:(BOOL)animated{
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    [self.navigationController setToolbarHidden:YES animated:YES];
}

- (void)viewDidUnload
{
    self.captureSession = nil;
    self.previewLayer = nil;
    self.cameraView = nil;
    [super viewDidUnload];
}

- (void) viewWillAppear:(BOOL)animated{
}

- (void) viewWillDisappear:(BOOL)animated{
    [self.captureSession stopRunning];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

#pragma mark - Getter / Setter

-(AVCaptureSession *)captureSession{
    if(!_captureSession){
        _captureSession = [[AVCaptureSession alloc] init];
    }
    return _captureSession;
}

-(AVCaptureVideoPreviewLayer *)previewLayer{
    if(!_previewLayer){
        _previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:[self captureSession]];
    }
    return _previewLayer;
}

-(UIView*) cameraView{
    if(!_cameraView){
        _cameraView = [[UIView alloc] initWithFrame:[[[UIApplication sharedApplication] delegate] window].frame];
    }
    return _cameraView;
}

#pragma mark - Multitouch handling

-(void) handleTap:(UITapGestureRecognizer*)sender{ 
    if(sender.state == UIGestureRecognizerStateEnded){
        [self.navigationController setNavigationBarHidden:!self.navigationController.navigationBar.hidden animated:YES];
        [self.navigationController setToolbarHidden:!self.navigationController.toolbar.hidden animated:YES];
    }
}

#pragma mark - AVCaptureVideoDataOutputSampleBufferDelegate

-(void) timerTask:(NSTimer*) t{
    [convolution setCanceled:YES];
    [timer invalidate];
}

-(void) configTimer{
    if(timer) [timer invalidate];
    timer = [NSTimer scheduledTimerWithTimeInterval:0.03 
                                             target:self 
                                           selector:@selector(timerTask:) 
                                           userInfo:nil 
                                            repeats:YES];
}

-(void) captureOutput:(AVCaptureOutput *)captureOutput 
didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer 
       fromConnection:(AVCaptureConnection *)connection 
{
    /*Récupération image*/
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer); 
    CVPixelBufferLockBaseAddress( imageBuffer, 0);
    imageSize.height = CVPixelBufferGetHeight(imageBuffer);
    imageSize.width = CVPixelBufferGetWidth(imageBuffer);
    uint8_t * imageData = CVPixelBufferGetBaseAddress(imageBuffer);
    
    /*LOGS*/
#ifdef DEBUG
    NSLog(@"ImageSize --> %f\t%f", imageSize.width, imageSize.height);
    if (!date) {
        date = [NSDate date];
    }
    else{
        NSDate * newDate = [NSDate date];
        
        NSLog(@"TEMPS ----> %f", [newDate timeIntervalSinceDate:date]);
        
        date = newDate;
    }
#endif

    
    /*Mise en place de l'image*/
    convolution.imageData = imageData;
    convolution.imageSize = imageSize;
    
    /*Création resut tab*/
    result = malloc(imageSize.width * imageSize.height * sizeof(int));
    convolution.result = result;
    
    [self performSelectorOnMainThread:@selector(configTimer) withObject:nil waitUntilDone:YES];
    
    [convolution setCanceled:NO];

    [convolution applyMask];
    NSLog(@"%d",convolution->nbPoint);
    free(result);
    
    /*Unlock base adress*/
    CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
}


@end
