//
//  MDVideoController.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 07/12/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "MDVideoController.h"
#import "MDAppDelegate.h"
#import "Frame.h"

@interface MDVideoController (private)

-(void) captureSessionInitialization;

@end

@implementation MDVideoController (private)

-(void)captureSessionInitialization{
    /*Input Device*/
    AVCaptureDeviceInput * captureInput = [AVCaptureDeviceInput 
										  deviceInputWithDevice:[AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo] 
										  error:nil];
    /*Output Device*/
    AVCaptureVideoDataOutput *captureOutput = [[AVCaptureVideoDataOutput alloc] init];
    captureOutput.alwaysDiscardsLateVideoFrames = NO; 
	//captureOutput.minFrameDuration = CMTimeMake(1, 10);
    
    /*We create a serial queue to handle the processing of our frames*/
	dispatch_queue_t queue;
	queue = dispatch_queue_create("cameraQueue",NULL);
	[captureOutput setSampleBufferDelegate:self.motionDetectionManager queue:queue];
	dispatch_release(queue);
    
    // Set the video output to store frame in BGRA (It is supposed to be faster)
	NSString* key = (NSString*) kCVPixelBufferPixelFormatTypeKey; 
	NSNumber* value = [NSNumber numberWithUnsignedInt:kCVPixelFormatType_32BGRA]; 
	NSDictionary* videoSettings = [NSDictionary dictionaryWithObject:value forKey:key]; 
	[captureOutput setVideoSettings:videoSettings];
    
    /*We add input and output*/
	[self.captureSession addInput:captureInput];
	[self.captureSession addOutput:captureOutput];
    
	/*We add the preview layer*/
	self.previewLayer = [AVCaptureVideoPreviewLayer layerWithSession: self.captureSession];
	self.previewLayer.frame = [[[UIApplication sharedApplication] delegate] window].frame;
	self.previewLayer.videoGravity =  AVLayerVideoGravityResizeAspectFill;   
    [self.cameraView.layer addSublayer: self.previewLayer];
    NSLog(@"%f\t%f", self.previewLayer.bounds.size.width, self.previewLayer.bounds.size.height);
    
}

@end


@implementation MDVideoController


@synthesize previewLayer = _previewLayer;
@synthesize captureSession = _captureSession;
@synthesize motionDetectionManager = _motionDetectionManager;
@synthesize cameraView = _cameraView;
@synthesize test;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[[[UIApplication sharedApplication] delegate] window] addSubview:self.cameraView];
    
    self.view.backgroundColor = [UIColor clearColor];
    
    [[(MDAppDelegate*)[[UIApplication sharedApplication]delegate] window] addSubview:self.navigationController.view];
    
    [self captureSessionInitialization];
    
    [[self captureSession] startRunning];
    
    UITapGestureRecognizer * tapGest = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap:)];
    tapGest.numberOfTapsRequired = 1;
    tapGest.delaysTouchesBegan = YES;
    [self.view addGestureRecognizer:tapGest];
    
    
    CGRect rect;
    rect.origin.x = 100;
    rect.origin.y = 100;
    rect.size.width = 10;
    rect.size.height=10;
     test = [[UIView alloc] initWithFrame:rect];
    [test setBackgroundColor:[UIColor redColor]];
    [self.view addSubview:test];

}
-(void)viewDidAppear:(BOOL)animated{
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    [self.navigationController setToolbarHidden:YES animated:YES];
}

- (void)viewDidUnload
{
    self.captureSession = nil;
    self.previewLayer = nil;
    self.cameraView = nil;
    self.motionDetectionManager = nil;
    [super viewDidUnload];
}

- (void) viewWillAppear:(BOOL)animated{
    [[self motionDetectionManager] startMotionDetection];
}

- (void) viewWillDisappear:(BOOL)animated{
    [self.captureSession stopRunning];
    [[self motionDetectionManager]stopMotionDetection];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

#pragma mark - Getter / Setter

-(AVCaptureSession *)captureSession{
    if(!_captureSession){
        _captureSession = [[AVCaptureSession alloc] init];
    }
    return _captureSession;
}

-(AVCaptureVideoPreviewLayer *)previewLayer{
    if(!_previewLayer){
        _previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:[self captureSession]];
    }
    return _previewLayer;
}

-(MotionDetectionManager *)motionDetectionManager{
    if(!_motionDetectionManager){
        _motionDetectionManager = [[MotionDetectionManager alloc] initWithProcessingQueueSize:500];
        [_motionDetectionManager setDelegate:self];
    }
    return _motionDetectionManager;
}

-(UIView*) cameraView{
    if(!_cameraView){
        _cameraView = [[UIView alloc] initWithFrame:[[[UIApplication sharedApplication] delegate] window].frame];
    }
    return _cameraView;
}

#pragma mark - Multitouch handling

-(void) handleTap:(UITapGestureRecognizer*)sender{ 
    if(sender.state == UIGestureRecognizerStateEnded){
        [self.navigationController setNavigationBarHidden:!self.navigationController.navigationBar.hidden animated:YES];
        [self.navigationController setToolbarHidden:!self.navigationController.toolbar.hidden animated:YES];
    }
}

#pragma mark - AVCaptureVideoDataOutputSampleBufferDelegate
-(void) captureOutput:(AVCaptureOutput *)captureOutput 
didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer 
       fromConnection:(AVCaptureConnection *)connection 
           pointInfos:(Point2D)point
{
    CGRect rect;
    rect.origin.x = (point.x * (double)(768.0 / (double)iPad2ScreenSizeX));
    rect.origin.y = (point.y * (double)(1024.0 / (double)iPad2ScreenSizeY));
    rect.size.width = 10;
    rect.size.height=10;
    
    if(!test){
        test = [[UIView alloc] initWithFrame:rect];
        [test setBackgroundColor:[UIColor redColor]];
    }
    else {
        [test performSelectorOnMainThread:@selector(removeFromSuperview) withObject:nil waitUntilDone:YES];
        test = [[UIView alloc] initWithFrame:rect];
        [test setBackgroundColor:[UIColor redColor]];
    }
    
    [self.view performSelectorOnMainThread:@selector(addSubview:) withObject:test waitUntilDone:YES];
    
    
}
@end
