 //
//  Frame.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 04/01/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#define MATRIX
#undef MATRIX
#import "Frame.h"
#import "MotionCapture.h"

@interface Frame (private)

-(void) configurePoints;

@end

@implementation Frame(private)

-(void) configurePoints{
    
    /*Longueur de la frame*/
    
    /*Largeur de la frame*/
}

@end

@implementation Frame

@synthesize motions = _motions;

@synthesize 
    xUserAcceleration=_xUserAcceleration, 
    yUserAcceleration=_yUserAcceleration, 
    zUserAcceleration=_zUserAcceleration;

@synthesize points=_points;

@synthesize startDate;

@synthesize frameInterval;

@synthesize attitudeRef;

@synthesize frameNumber;

@synthesize plane = _plane;


- (id)init {
    self = [super init];
    if (self) {
        
        _xUserAcceleration =0.0;
        _yUserAcceleration=0.0;
        _zUserAcceleration=0.0;
        
        _points = malloc(4*sizeof(Point3D));

    }
    return self;
}

-(id) initWithFrameNumber:(NSUInteger)n imageBufferRef:(CVImageBufferRef)imageBuffer andStartDate:(NSDate*)date{
    self = [super init];
    if(self){
        frameNumber = n;
        imageBufferRef = imageBuffer;
        startDate = date;

        _xUserAcceleration = 0.0;
        _yUserAcceleration = 0.0;
        _zUserAcceleration = 0.0;
        
        
        _points = malloc(4 * sizeof(Point3D) );
    }
    return self;
}

- (Plane3D) plane{
    return _plane;
}

-(NSMutableArray *)motions{
    if(!_motions){
        _motions = [[NSMutableArray alloc]init];
    }
    return _motions;
}

-(void)addCaptureMotion:(MotionCapture *)motion{
    if (motion.attitude.roll && motion.attitude.pitch && motion.attitude.yaw) {
        [[self motions] addObject:motion];
        if(!attitudeRef)
            attitudeRef = motion.attitude;
    }
}

-(CVImageBufferRef) imageBufferRef{
    return imageBufferRef;
}

-(void) calculFramePosition:(Frame*)previousFrame{
    
    double norme;
    
    if ([[self motions] count] == 0){
        return;
    }
    
    
    if(!previousFrame || !previousFrame->framePositionProcessed){
        /*Initialisation de la position de la frame*/
        _points[0].x = P1X;
        _points[0].y = P1Y;
        _points[0].z = P1Z;
        
        _points[1].x = P2X;
        _points[1].y = P2Y;
        _points[1].z = P2Z;
        
        _points[2].x = P3X;
        _points[2].y = P3Y;
        _points[2].z = P3Z;
        
        _points[3].x = P4X;
        _points[3].y = P4Y;
        _points[3].z = P4Z;

    
    }
    else {
        /*Calcul par rapport à la frame précédente*/
        if ([_motions count] < 2) {
            return;
        }
        
        /*Récupération de l'attitude*/
        CMAttitude* attitude = ((MotionCapture *)[_motions objectAtIndex:1]).attitude;
        
        /*Calcul du delta attitude*/
        [attitude multiplyByInverseOfAttitude:previousFrame.attitudeRef];
        
        
        if ( ABS(attitude.yaw) < SEUIL_DELTA && ABS(attitude.pitch) < SEUIL_DELTA && ABS(attitude.roll) < SEUIL_DELTA) {
            /*Seuil (variation du gyroscope)*/
            _points[0] = previousFrame.points[0];
            
            _points[1] = previousFrame.points[1];
            
            _points[2] = previousFrame.points[2];
            
            _points[3] = previousFrame.points[3];
        }
        else {
            /*Récupération de la matrice de rotation*/
            CMRotationMatrix matrix = attitude.rotationMatrix;

            /*Calcul du positionnement de la frame dans l'espace 3D*/
            _points[0].x = previousFrame.points[0].x*matrix.m11 + previousFrame.points[0].y*matrix.m12 + previousFrame.points[0].z*matrix.m13;
            _points[0].y = previousFrame.points[0].x*matrix.m21 + previousFrame.points[0].y*matrix.m22 + previousFrame.points[0].z*matrix.m23;
            _points[0].z = previousFrame.points[0].x*matrix.m31 + previousFrame.points[0].y*matrix.m32 + previousFrame.points[0].z*matrix.m33;
        
            _points[1].x = previousFrame.points[1].x*matrix.m11 + previousFrame.points[1].y*matrix.m12 + previousFrame.points[1].z*matrix.m13;
            _points[1].y = previousFrame.points[1].x*matrix.m21 + previousFrame.points[1].y*matrix.m22 + previousFrame.points[1].z*matrix.m23;
            _points[1].z = previousFrame.points[1].x*matrix.m31 + previousFrame.points[1].y*matrix.m32 + previousFrame.points[1].z*matrix.m33;
        
            _points[2].x = previousFrame.points[2].x*matrix.m11 + previousFrame.points[2].y*matrix.m12 + previousFrame.points[2].z*matrix.m13;
            _points[2].y = previousFrame.points[2].x*matrix.m21 + previousFrame.points[2].y*matrix.m22 + previousFrame.points[2].z*matrix.m23;
            _points[2].z = previousFrame.points[2].x*matrix.m31 + previousFrame.points[2].y*matrix.m32 + previousFrame.points[2].z*matrix.m33;
        
            _points[3].x = previousFrame.points[3].x*matrix.m11 + previousFrame.points[3].y*matrix.m12 + previousFrame.points[3].z*matrix.m13;
            _points[3].y = previousFrame.points[3].x*matrix.m21 + previousFrame.points[3].y*matrix.m22 + previousFrame.points[3].z*matrix.m23;
            _points[3].z = previousFrame.points[3].x*matrix.m31 + previousFrame.points[3].y*matrix.m32 + previousFrame.points[3].z*matrix.m33;
        }

    }
    
    /*Origine du plan*/
    _plane.origin = _points[0];
    
    /*Vecteurs*/
    _plane.vectors[0].x = _points[1].x - _points[0].x;
    _plane.vectors[0].y = _points[1].y - _points[0].y;
    _plane.vectors[0].z = _points[1].z - _points[0].z;
    
    _plane.vectors[1].x = _points[2].x - _points[0].x;
    _plane.vectors[1].y = _points[2].y - _points[0].y;
    _plane.vectors[1].z = _points[2].z - _points[0].z;
   
    /*Vecteurs unitaires*/
    norme = sqrt(pow(_plane.vectors[0].x, 2) + pow(_plane.vectors[0].y, 2) + pow(_plane.vectors[0].z, 2));
    _plane.normalizedVectors[0].x = _plane.vectors[0].x / norme;
    _plane.normalizedVectors[0].y = _plane.vectors[0].y / norme;
    _plane.normalizedVectors[0].z = _plane.vectors[0].z / norme;
    
    norme = sqrt(pow(_plane.vectors[1].x, 2) + pow(_plane.vectors[1].y, 2) + pow(_plane.vectors[1].z, 2));
    _plane.normalizedVectors[1].x = _plane.vectors[1].x / norme;
    _plane.normalizedVectors[1].y = _plane.vectors[1].y / norme;
    _plane.normalizedVectors[1].z = _plane.vectors[1].z / norme;
    
    /*Indique que la frame est traitée*/
    framePositionProcessed = YES;
}

@end
