//
//  FrameIntersectionManager.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 08/03/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "FrameIntersectionManager.h"



@implementation FrameIntersectionManager

@synthesize firstFrame, 
            secondFrame, 
            medianPlane=_medianPlane, 
            projectedVertices2D=_projectedVertices2D, 
            projectedLines2D=_projectedLines2D,
            intersectVertices = _intersectVertices,
            normal = _normal,
            intersectVerticesNumber;


- (Vector3D*) normal{
    if(!_normal){
        _normal = malloc(sizeof(Vector3D));
        _normal->x = (self.medianPlane->vectors[0].y * _medianPlane->vectors[1].z) - (_medianPlane->vectors[0].z * _medianPlane->vectors[1].y);
        _normal->y = (self.medianPlane->vectors[0].z * _medianPlane->vectors[1].x) - (_medianPlane->vectors[0].x * _medianPlane->vectors[1].z);
        _normal->z = (self.medianPlane->vectors[0].x * _medianPlane->vectors[1].y) - (_medianPlane->vectors[0].y * _medianPlane->vectors[1].x);
    }
    return _normal;
}

- (Plane3D *)medianPlane{
    if(!_medianPlane){
        /*CREATION DU PLAN*/
        _medianPlane = malloc(sizeof(Plane3D));

        _medianPlane->origin.x = (firstFrame.plane.origin.x + secondFrame.plane.origin.x) / 2.0; 
        _medianPlane->origin.y = (firstFrame.plane.origin.y + secondFrame.plane.origin.y) / 2.0; 
        _medianPlane->origin.z = (firstFrame.plane.origin.z + secondFrame.plane.origin.z) / 2.0;
        
        _medianPlane->vectors[0].x = (firstFrame.plane.vectors[0].x + secondFrame.plane.vectors[0].x) / 2.0; 
        _medianPlane->vectors[0].y = (firstFrame.plane.vectors[0].y + secondFrame.plane.vectors[0].y) / 2.0; 
        _medianPlane->vectors[0].z = (firstFrame.plane.vectors[0].z + secondFrame.plane.vectors[0].z) / 2.0; 
        
        _medianPlane->vectors[1].x = (firstFrame.plane.vectors[1].x + secondFrame.plane.vectors[1].x) / 2.0; 
        _medianPlane->vectors[1].y = (firstFrame.plane.vectors[1].y + secondFrame.plane.vectors[1].y) / 2.0; 
        _medianPlane->vectors[1].z = (firstFrame.plane.vectors[1].z + secondFrame.plane.vectors[1].z) / 2.0; 
        
        double tmp = sqrt(pow(_medianPlane->vectors[0].x, 2)+ pow(_medianPlane->vectors[0].y, 2) + pow(_medianPlane->vectors[0].z, 2));
        _medianPlane->vectors[0].x /= tmp;
        _medianPlane->vectors[0].y /= tmp;
        _medianPlane->vectors[0].z /= tmp;
        
        tmp = sqrt(pow(_medianPlane->vectors[1].x, 2)+ pow(_medianPlane->vectors[1].y, 2) + pow(_medianPlane->vectors[1].z, 2));
        _medianPlane->vectors[1].x /= tmp;
        _medianPlane->vectors[1].y /= tmp;
        _medianPlane->vectors[1].z /= tmp;

    }
    return _medianPlane;
}

/**
 ** projectedVertices2D                         p1  ------- p2
 ** |                                               |     |
 ** -->[0] --> First frame vertices                 |     |
 ** |   |                                           |     |
 ** |   ------> [0] p1                          p3  ------- p4
 ** |   ------> [1] p2                    
 ** |   ------> [2] p3
 ** |   ------> [3] p4
 ** -->[1] --> Second frame vertices
 **     |
 **     ------> [0] p1
 **     ------> [1] p2
 **     ------> [2] p3
 **     ------> [3] p4
 **/
- (Point2D**) projectedVertices2D{
    if(!_projectedVertices2D){
        /*Projection des points sur le plan médian*/
        _projectedVertices2D = malloc(2*sizeof(Point2D*));
        
        /*constantes*/
        //double nxOx = self.normal->x * self.medianPlane->origin.x;
        //ouble nyOy = _normal->y * _medianPlane->origin.y;
        //ouble nzOz = _normal->z * _medianPlane->origin.z;
      //  
        //double nxnynz = pow(_normal->x, 2) + pow(_normal->y, 2) + pow(_normal->z, 2);
        
        /*Frame 1*/
        for (int i=0; i<4; i++) {
            //double alpha = _normal->x * firstFrame.points[i].x - nxOx;
            //double beta = _normal->y * firstFrame.points[i].y - nyOy;
            //double gamma = _normal->z * firstFrame.points[i].z - nzOz;
            
            //double k = - (alpha + beta + gamma) / nxnynz;
            
            _projectedVertices2D[0] = malloc(4*sizeof(Point2D));
            
        }
        
        /*Frame 2*/
    }
    return _projectedVertices2D;
}

/**
 ** The usage is the same of projectedVertices2D :: ---> Line2D[2][4]
 **/
- (Line2D**) projectedLines2D{
    if(!_projectedLines2D){
        /*Calcul des lignes*/
    }
    return _projectedLines2D;
}


- (id)initWithFirstFrame:(Frame*)first andSecondFrame:(Frame*)second{
    self = [super init];
    if (self) {
        firstFrame = first;
        secondFrame = second;
    }
    return self;
}

- (Point3D *)intersectFrames{
    
    /*Création du plan*/
    /*Projection*/
    
    /*Limites frame 2*/
    double xMin, xMax, yMax, yMin;
    for (int i = 0; i < 4; i++) {
        if(self.projectedVertices2D[1][i].x >= xMax) xMax = self.projectedVertices2D[1][i].x;
        if(self.projectedVertices2D[1][i].y >= yMax) yMax = self.projectedVertices2D[1][i].y;
        if(self.projectedVertices2D[1][i].x <= xMin) xMin = self.projectedVertices2D[1][i].x;
        if(self.projectedVertices2D[1][i].y <= yMin) yMin = self.projectedVertices2D[1][i].y;
    }
    
    /*Pour toutes les lignes de la frame 1*/
    for (int i=0; i < 4; i++) {
        /*Pour toutes les lignes de la frame 2*/
        for (int j = 0; j <4; j++) {
            Point2D * p = [self intersectLine:&self.projectedLines2D[0][i] withLine:&self.projectedLines2D[0][j]];
            if(   p    != nil
               && p->x <= xMax 
               && p->x >= xMin 
               && p->y <= yMax 
               && p->y >= yMin )
            {
                intersectVerticesNumber++;
                if(_intersectVertices){
                    _intersectVertices = realloc(_intersectVertices, intersectVerticesNumber * sizeof(Point3D));
                }
                else{
                    _intersectVertices = malloc(intersectVerticesNumber * sizeof(Point3D));
                }
                
                /*Ajout du point sans le vecteur*/
//                _intersectVertices[intersectVerticesNumber-1] = point
            }
        }
    }
    
    /* Retro projection */
    
    
    return nil;
}

- (Point2D*) intersectLine:(Line2D*)first withLine:(Line2D*)second{
    Point2D * point = malloc(sizeof(Point2D));
    double tmp = first->a - second->a == 0;
    if(tmp == 0){
        return nil;
    }
    point->x = (second->b - first->b)/tmp;
    point->y = first->a * point->x + first->b;
    return point;
}

-(Point2D*) projectPoint:(Point2D)pointInFrameCoordinates{
    if(secondFrame == nil || firstFrame == nil) return nil;
    
    double norm = sqrt(pow(firstFrame.plane.vectors[0].x, 2) + pow(firstFrame.plane.vectors[0].y, 2) + pow(firstFrame.plane.vectors[0].z, 2));
    Vector3D if1 = {firstFrame.plane.vectors[0].x/norm,firstFrame.plane.vectors[0].y/norm,firstFrame.plane.vectors[0].z/norm};
    norm = sqrt(pow(firstFrame.plane.vectors[1].x, 2) + pow(firstFrame.plane.vectors[1].y, 2) + pow(firstFrame.plane.vectors[1].z, 2));
    Vector3D jf1 = {firstFrame.plane.vectors[1].x/norm,firstFrame.plane.vectors[1].y/norm,firstFrame.plane.vectors[1].z/norm};
    
    norm = sqrt(pow(secondFrame.plane.vectors[0].x, 2) + pow(secondFrame.plane.vectors[0].y, 2) + pow(secondFrame.plane.vectors[0].z, 2));
    Vector3D if2 = {secondFrame.plane.vectors[0].x/norm,secondFrame.plane.vectors[0].y/norm,secondFrame.plane.vectors[0].z/norm};
    norm = sqrt(pow(secondFrame.plane.vectors[1].x, 2) + pow(secondFrame.plane.vectors[1].y, 2) + pow(secondFrame.plane.vectors[1].z, 2));
    Vector3D jf2 = {secondFrame.plane.vectors[1].x/norm,secondFrame.plane.vectors[1].y/norm,secondFrame.plane.vectors[1].z/norm};
    
    NSLog(@"Firstframe plane --> %f \t %f \t %f", firstFrame.plane.origin.x, firstFrame.plane.origin.y, firstFrame.plane.origin.z);
    NSLog(@"\t V1 --> %f \t %f \t %f",firstFrame.plane.vectors[0].x, firstFrame.plane.vectors[0].y, firstFrame.plane.vectors[0].z);
    NSLog(@"\t V2 --> %f \t %f \t %f",firstFrame.plane.vectors[1].x, firstFrame.plane.vectors[1].y, firstFrame.plane.vectors[1].z);
    for (int i = 0; i < 4; i++) {
        NSLog(@"\t P%d --> %f \t %f \t %f",i,firstFrame.points[i].x, firstFrame.points[i].y, firstFrame.points[i].z);
    }
    NSLog(@"SecondPlane plane --> %f \t %f \t %f", secondFrame.plane.origin.x, secondFrame.plane.origin.y, secondFrame.plane.origin.z);
    NSLog(@"\t V1 --> %f \t %f \t %f",secondFrame.plane.vectors[0].x, secondFrame.plane.vectors[0].y, secondFrame.plane.vectors[0].z);
    NSLog(@"\t V2 --> %f \t %f \t %f",secondFrame.plane.vectors[1].x, secondFrame.plane.vectors[1].y, secondFrame.plane.vectors[1].z);
    for (int i = 0; i < 4; i++) {
        NSLog(@"\t P%d --> %f \t %f \t %f",i,secondFrame.points[i].x, secondFrame.points[i].y, secondFrame.points[i].z);
    }
    NSLog(@"MedianPlane --> %f \t %f \t %f", self.medianPlane->origin.x, _medianPlane->origin.y, _medianPlane->origin.z);
    NSLog(@"\t V1 --> %f \t %f \t %f",_medianPlane->vectors[0].x, _medianPlane->vectors[0].y, _medianPlane->vectors[0].z);
    NSLog(@"\t V2 --> %f \t %f \t %f",_medianPlane->vectors[1].x, _medianPlane->vectors[1].y, _medianPlane->vectors[1].z);
    NSLog(@"\t Normal Vector --> %f \t %f \t %f", self.normal->x, _normal->y, _normal->z );
    
    
    /*Projection du point sur le plan médian*/
    
    /*Restitution des coordonnées du point 2D en 3D*/
    
    Point3D point;
    point.x = pointInFrameCoordinates.x * if1.x + pointInFrameCoordinates.y * jf1.x;
    point.y = pointInFrameCoordinates.x * if1.y + pointInFrameCoordinates.y * jf1.y;
    point.z = pointInFrameCoordinates.x * if1.z + pointInFrameCoordinates.y * jf1.z;
    
    /*point.x = pointInFrameCoordinates.x * firstFrame.plane.vectors[0].x + pointInFrameCoordinates.y * firstFrame.plane.vectors[1].x;
    point.y = pointInFrameCoordinates.x * firstFrame.plane.vectors[0].y + pointInFrameCoordinates.y * firstFrame.plane.vectors[1].y;
    point.z = pointInFrameCoordinates.x * firstFrame.plane.vectors[0].z + pointInFrameCoordinates.y * firstFrame.plane.vectors[1].z;*/
    NSLog(@"Point 2D (10, 10) en 3D --> %f \t %f \t %f",point.x, point.y, point.z);

    
/*Projection F1 --> Plan*/
    /*constantes*/
    double nxOx = self.normal->x * self.medianPlane->origin.x;
    double nyOy = _normal->y * _medianPlane->origin.y;
    double nzOz = _normal->z * _medianPlane->origin.z;
    double nxnynz = pow(_normal->x, 2) + pow(_normal->y, 2) + pow(_normal->z, 2);
    
    /*Frame 1*/
    double alpha = _normal->x * point.x - nxOx;
    double beta = _normal->y *  point.y - nyOy;
    double gamma = _normal->z * point.z - nzOz;
    double k = - (alpha + beta + gamma) / nxnynz;
    Point3D planeProjectedPoint = {k*_normal->x, k*_normal->y, k*_normal->z};
    NSLog(@"Point 3D projeté sur le plan médian --> %f \t %f \t %f",planeProjectedPoint.x, planeProjectedPoint.y, planeProjectedPoint.z);

/*Projection Plane --> F2*/
    
    /*Vecteur normal au plan de la frame 2*/
    Vector3D normalF2;
    normalF2.x = 
        (self.secondFrame.plane.vectors[0].y * secondFrame.plane.vectors[1].z) 
        - (secondFrame.plane.vectors[0].z * secondFrame.plane.vectors[1].y);
    normalF2.y = 
        (self.secondFrame.plane.vectors[0].z * secondFrame.plane.vectors[1].x) 
        - (secondFrame.plane.vectors[0].x * secondFrame.plane.vectors[1].z);
    normalF2.z = 
        (self.secondFrame.plane.vectors[0].x * secondFrame.plane.vectors[1].y) 
        - (secondFrame.plane.vectors[0].y * secondFrame.plane.vectors[1].x);
    
    nxOx = normalF2.x * secondFrame.plane.origin.x;
    nyOy = normalF2.y * secondFrame.plane.origin.y;
    nzOz = normalF2.z * secondFrame.plane.origin.z;
    nxnynz = pow(normalF2.x, 2) + pow(normalF2.y, 2) + pow(normalF2.z, 2);
    alpha = normalF2.x * planeProjectedPoint.x - nxOx;
    beta =  normalF2.y * planeProjectedPoint.y - nyOy;
    gamma = normalF2.z * planeProjectedPoint.z - nzOz;
    k = - (alpha + beta + gamma) / nxnynz;
    
    Point3D frameProjectedPoint = {
        k * normalF2.x, 
        k * normalF2.y, 
        k * normalF2.z
    };
    NSLog(@"Point 3D projeté sur le la seconde frame --> %f \t %f \t %f",frameProjectedPoint.x, frameProjectedPoint.y, frameProjectedPoint.z);
    /*Transformation du point 3D en 2D*/
    //Vector3D * i = &secondFrame.plane.vectors[0];
    //Vector3D * j = &secondFrame.plane.vectors[1];
    Point2D * finalPoint = malloc(sizeof(Point2D));
    finalPoint->y = (frameProjectedPoint.y * if2.x - frameProjectedPoint.x * if2.y) / (jf2.y * if2.x - jf2.x * if2.y);
    finalPoint->x = (frameProjectedPoint.x - finalPoint->y * jf2.x) / if2.x;
    NSLog(@"Point 3D projeté sur le la seconde frame --> %f \t %f \t %f",finalPoint->x * if2.x + finalPoint->y * if2.x, finalPoint->x * if2.y + finalPoint->y * if2.y, finalPoint->x * if2.z + finalPoint->y * if2.z);
    
    return finalPoint;
}
@end
