//
//  SimpleConvolutionMask.m
//  MotionDetection
//
//  Created by Kévin Toublanc on 10/04/12.
//  Copyright (c) 2012 Polytech'Tours. All rights reserved.
//

#import "SimpleConvolutionMask.h"
#import "NSException+Custom.h"

@implementation SimpleConvolutionMask

@synthesize imageData=_imageData;
@synthesize imageSize=_imageSize;
@synthesize result=_result;
@synthesize delegate=_delegate;
@synthesize mask=_mask;
@synthesize maskSize=_maskSize;
@synthesize canceled;

-(id<ConvolutionMaskProtocol>) initWithImage:   (uint8_t*)     image 
                                    imageSize:  (CGSize)       imageSize 
                                 resultImage:   (int*)     resultImage 
                                        mask:   (int*)     mask 
                                    maskSize:   (CGSize)       maskSize
                                    delegate:   (NSObject<ConvolutionMaskDelegate>*) delegate
{
    self = [super init];
    if (self) {
        _imageData  = image;
        _imageSize  = imageSize;
        _result     = resultImage;
        _delegate   = delegate;
        _mask       = mask;
        _maskSize   = maskSize;
        lut = getLutMatrix(0.33, 0.33, 0.34);
        canceled = NO;
    }
    return self;
}


-(void)applyMask
{
    halfMaskHeight = self.maskSize.height /2;
    halfMaskWidth = self.maskSize.width / 2;
    nbPixelsMask = self.maskSize.width * self.maskSize.height;
    for (int i = 0; i < self.imageSize.height; i++) {
        for (int j = 0; j < self.imageSize.width; j++) {
            if(canceled) return;
            [self applyMaskOnPixelonLine:i OnColumn:j];
            /*unsigned int debIMask=0, debJMask=0;
            unsigned int finIMask=0, finJMask=0;
            //indice I --> mask
            if(i < halfMaskHeight){
                debIMask = halfMaskHeight - i;
                finIMask = self.maskSize.height;
            }
            else{
                debIMask = 0;
                finIMask = (  self.imageSize.height - i > halfMaskHeight)
                            ? self.maskSize.height 
                            : halfMaskHeight +  self.imageSize.height - i;
            }
            
            //indice j --> mask
            if(j < halfMaskWidth){
                debJMask = (halfMaskWidth) - j;
                finJMask = self.maskSize.width;
            }
            else{
                debJMask = 0;
                finJMask = (  self.imageSize.width - j > (halfMaskWidth))
                ? self.maskSize.width 
                : halfMaskWidth +  self.imageSize.width - j;
            }
            
        
            //Application du mask sur le pixel i
            int res=0;
            for(int iMask = debIMask; iMask < finIMask; iMask++){
                for (int jMask = debJMask; jMask < finJMask; jMask++) {

                    int iImage = i - (halfMaskHeight - iMask);
                    int jImage = j - (halfMaskWidth - jMask);
                    int b = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage))];
                    int g = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage)+1)];
                    int r = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage)+2)];
                    res +=  self.mask[(int)(iMask * self.maskSize.width + jMask)] * lut[(int)b][(int)g][(int)r];
                    
                }
            }
            self.result[(int)(i * self.imageSize.width + j)] = res / nbPixelsMask;*/
        }
    }
}

-(void)applyMaskOnPixel:(unsigned int)pixelNumber{
    @throw [NSException NotImplementedException];
}

-(void)applyMaskOnPixelonLine:(unsigned int)line OnColumn:(unsigned int)column{
    unsigned int debIMask=0, debJMask=0;
    unsigned int finIMask=0, finJMask=0;
    /*indice I --> mask*/
    if(line < halfMaskHeight){
        debIMask = halfMaskHeight - line;
        finIMask = self.maskSize.height;
    }
    else{
        debIMask = 0;
        finIMask = (  self.imageSize.height - line > halfMaskHeight)
        ? self.maskSize.height 
        : halfMaskHeight +  self.imageSize.height - line;
    }
    
    //indice j --> mask
    if(column < halfMaskWidth){
        debJMask = (halfMaskWidth) - column;
        finJMask = self.maskSize.width;
    }
    else{
        debJMask = 0;
        finJMask = (  self.imageSize.width - column > (halfMaskWidth))
        ? self.maskSize.width 
        : halfMaskWidth +  self.imageSize.width - column;
    }
    
    
    /*Application du mask sur le pixel i*/
    int res=0;
    for(int iMask = debIMask; iMask < finIMask; iMask++){
        for (int jMask = debJMask; jMask < finJMask; jMask++) {
            
            int iImage = line - (halfMaskHeight - iMask);
            int jImage = column - (halfMaskWidth - jMask);
            int b = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage))];
            int g = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage)+1)];
            int r = self.imageData[(int)(NB_OCTET_PAR_PIXEL * (iImage * self.imageSize.width + jImage)+2)];
            res +=  self.mask[(int)(iMask * self.maskSize.width + jMask)] * lut[(int)b][(int)g][(int)r];
            
        }
    }
    self.result[(int)(line * self.imageSize.width + column)] = res / nbPixelsMask;
    
    //Detection point d'intéret
    if (column > 0 && (self.result[(int)(line * self.imageSize.width + column-1)]*res) < 0) {
        NSLog(@"point d'interret");
    }
}

@end
