//
//  MotionDetectionManager.h
//  MotionDetection
//
//  Created by Kevin Toublanc on 04/01/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreMotion/CoreMotion.h>
#import <AVFoundation/AVFoundation.h>
#import "CircularFrameProcessingQueue.h"
#import "Geometry.h"

#define MotionDetectionManagerMaximalFrameNumber            100000
#define MotionDetectionManagerFrameProcessingQueueSize      80
#define MotionDetectionManagerMotionCaptureInterval         0.01
#define MotionDetectionManagerCMMotionManagerUpdateInterval 0.005
#define MotionDetectionManagerMotionQueueName               @"Motion Detection Queue"

@protocol MotionDetectionManagerDelegate <NSObject>
@required
-(void) captureOutput:(AVCaptureOutput *)captureOutput 
didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer 
       fromConnection:(AVCaptureConnection *)connection pointInfos:(Point2D)point;

@end



@interface MotionDetectionManager : NSObject <AVCaptureVideoDataOutputSampleBufferDelegate>{
    @private
    /*Queue de traitement*/
    CircularFrameProcessingQueue * frameProcessingQueue;
    
    /*Motion manager*/
    CMMotionManager *manager;
    
    /*Current frame number*/
    NSUInteger currentFrameNumber;
    
    /*Motion Capture Time Interval*/
    NSTimeInterval motionCaptureInterval;
    
    /*Boolean indicating that the object runs motion detection algorithm*/
    BOOL isMotionDetecting;
    
    /*Motion Timer*/
    NSTimer* motionTimer;
    
    NSOperationQueue * motionQueue;
    
    
}

@property (nonatomic, weak) id<MotionDetectionManagerDelegate> delegate;

@property (nonatomic) Point2D point;
@property (nonatomic) Point3D pointIn3D;

-(id)initWithProcessingQueueSize:(NSUInteger)size;

-(void)stopMotionDetection;

-(void) startMotionDetection;

-(void) saveProcessingQueueToFile;

@end
