//
//  MotionDetectionManager.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 04/01/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MotionDetectionManager.h"
#import "CircularFrameProcessingQueue.h"
#import "Frame.h"
#import "MotionCapture.h"
#import "MotionMovementCalculOperation.h"
#import "NSException+Custom.h"
#import "FrameIntersectionManager.h"
#import "Geometry.h"
@interface MotionDetectionManager (private)

/*Lance la capture des mouvements de l'iPad*/
-(void) beginDeviceMotionCapture;

-(void) newFrame:(CVImageBufferRef)imageBufferRef;

@end

@implementation MotionDetectionManager (private)

-(void) addNewMotionCaptureToFrame:(NSTimer*)timer{
    [frameProcessingQueue addMotionCaptureToCurrentFrame:[[MotionCapture alloc] initWithDeviceMotion:manager.deviceMotion]];
}

-(void) beginDeviceMotionCapture{
    if(motionTimer){
        [motionTimer invalidate];
    }
    motionTimer = [NSTimer scheduledTimerWithTimeInterval:MotionDetectionManagerMotionCaptureInterval 
                                                   target:self 
                                                 selector:@selector(addNewMotionCaptureToFrame:) 
                                                 userInfo:nil 
                                                  repeats:YES];
}

/**
 * Create new Frame, initialize it and begin motion capture for the current frame
 *
 */
-(void)newFrame:(CVImageBufferRef)imageBufferRef{
    if (currentFrameNumber > MotionDetectionManagerMaximalFrameNumber) {
        currentFrameNumber = 0;
    }
    Frame * frame = [[Frame alloc] initWithFrameNumber:currentFrameNumber imageBufferRef:imageBufferRef andStartDate:[[NSDate alloc] initWithTimeIntervalSinceNow:0]];
    currentFrameNumber++;
    [self performSelectorOnMainThread:@selector(beginDeviceMotionCapture) withObject:nil waitUntilDone:YES];
    Frame * currentFrame = [frameProcessingQueue currentFrame];
    [currentFrame setFrameInterval:[currentFrame.startDate timeIntervalSinceNow]];
    [frameProcessingQueue addFrame:frame];
}

@end



@implementation MotionDetectionManager

@synthesize delegate;
@synthesize point;
@synthesize pointIn3D;
-(id)initWithProcessingQueueSize:(NSUInteger)size{
    self = [super init];
    if(self){
        manager = [[CMMotionManager alloc] init];
        frameProcessingQueue = [[CircularFrameProcessingQueue alloc] initWithSize:size];
        currentFrameNumber = 0;
        point.x = 400.0 * iPad2ScreenSizeX / 768.0;
        point.y = 400.0 * iPad2ScreenSizeY / 1024.0;
        
        motionQueue = [[NSOperationQueue alloc] init];
        [motionQueue setName:@"MotionDetectionQueue"];
    }
    return self;
}

-(void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection{
    /*Contrôle du lancement de la capture*/
    if(!isMotionDetecting)return;
    
    /*Contrôle de la présence de la queue de traitement*/
    if(!motionQueue)@throw [NSException PropertyNotAssignedException:@"MotionQueue"];
    
    /*Invalidation du timer*/
    if(motionTimer){
        [motionTimer invalidate];
        motionTimer = nil;
    }

    
    /*Convertion de l'image*/
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
        
    /*Création de la frame et ajout dans la queue circulaire*/
    [self newFrame:imageBuffer];
    
    
    if([frameProcessingQueue previousFrame] && [frameProcessingQueue previousFrame:2]){
        
                
        Frame* firstFrame = [frameProcessingQueue previousFrame:2];
        Frame* secondFrame = [frameProcessingQueue previousFrame];
        
        
        [secondFrame calculFramePosition:firstFrame];
        
        //if(firstFrame.plane.origin.x == 0.0 && firstFrame.plane.origin.y == 0.0 && firstFrame.plane.origin.z == 0.0) return;
        //if(firstFrame.plane.vectors[0].x == 0.0 && firstFrame.plane.vectors[0].y == 0.0 && firstFrame.plane.vectors[0].z == 0.0) return;
        if([firstFrame.motions count] == 0 || [secondFrame.motions count]==0) return;

        /*Plan médian*/
        Plane3D medianPlane = MDMedianPlane(firstFrame.plane, secondFrame.plane);
        
        /*Normal plan médian*/
        Vector3D medianPlaneNormal = MDNormalVectorFromPlane(medianPlane);
        
        /*Normal seconde frame*/
        Vector3D secondFrameNormal = MDNormalVectorFromPlane(secondFrame.plane);
        
        /*Point point dans le repere 3D*/
        /*Point3D origin = firstFrame.plane.origin;
        if(origin.x == 0 && origin.y == 0 && origin.z == 0)
            NSLog(@"ZERO");
        Point3D p1 = firstFrame.points[1];
        Vector3D v1 = firstFrame.plane.normalizedVectors[0];
        Vector3D v2 = firstFrame.plane.normalizedVectors[1];*/
        
        if((pointIn3D.x == 0.0 && pointIn3D.y == 0.0 && pointIn3D.z == 0.0)){
            pointIn3D.x = point.x * firstFrame.plane.normalizedVectors[0].x + point.y * firstFrame.plane.normalizedVectors[1].x + firstFrame.plane.origin.x;
            pointIn3D.y = point.x * firstFrame.plane.normalizedVectors[0].y + point.y * firstFrame.plane.normalizedVectors[1].y + firstFrame.plane.origin.y;
            pointIn3D.z = point.x * firstFrame.plane.normalizedVectors[0].z + point.y * firstFrame.plane.normalizedVectors[1].z + firstFrame.plane.origin.z;
        }
        
        
        /*Projection du point sur le plan médian*/
        Point3D medianPlanePoint = MDProjectPointOntoPlaneWhithOrthogonalProjection(pointIn3D, medianPlane, medianPlaneNormal);

        /*Projection de la projection sur la seconde frame*/
        Point3D secondFramePointIn3D = MDProjectPointOntoPlaneWhithOrthogonalProjection(medianPlanePoint, secondFrame.plane, secondFrameNormal);
        
        /*Calcul des coordonnées 2D du point 3D*/
        Point2D projectedPoint = MDTransform3DPointIn2DPoint(secondFramePointIn3D, secondFrame.plane);
        
        //pointIn3D = secondFramePointIn3D;
        
        //point = projectedPoint;
        
        [delegate captureOutput:captureOutput didOutputSampleBuffer:sampleBuffer fromConnection:connection pointInfos:projectedPoint];
        //NSLog(@"%f\t%f",projectedPoint.x, projectedPoint.y);
        //NSLog(@"%f\t%f\t%f",secondFramePointIn3D.x, secondFramePointIn3D.y,secondFramePointIn3D.z);
        //NSLog(@"%@", point);
    }

    
    /*Calcul du mouvement*/
    //MotionMovementCalculOperation * operation = [[MotionMovementCalculOperation alloc] initWithFrameProcessingQueue:frameProcessingQueue];
    //[operation addObserver:self forKeyPath:@"isFinished" options:0 context:NULL];
    //[motionQueue addOperation:operation];


}

-(void) startMotionDetection{
    /*Indication de l'état de la classe*/
    isMotionDetecting = YES;
    
    /*Contrôle de la présence de la queue de traitement*/
    if(!motionQueue)@throw [NSException PropertyNotAssignedException:@"MotionQueue"];
    
    /*Configuration de l'acquisition des mouvements*/
    [manager setDeviceMotionUpdateInterval:MotionDetectionManagerCMMotionManagerUpdateInterval];
    [manager startDeviceMotionUpdatesUsingReferenceFrame:CMAttitudeReferenceFrameXArbitraryCorrectedZVertical];//CMAttitudeReferenceFrameXArbitraryZVertical];
}

-(void) stopMotionDetection{
    /*Invalidation du timer*/
    if(motionTimer){
        [motionTimer invalidate];
        motionTimer = nil;
    }
    
    /*Indication de l'état de la classe*/
    isMotionDetecting = NO;
    
    /*Invalidation de l'acquisition des mouvements*/
    [manager stopDeviceMotionUpdates];
    
    /*Sauvegarde des acquisition dans un fichier*/
    //[self saveProcessingQueueToFile];
    
    /*Annulation des opération en cours dans la queue*/
    [motionQueue cancelAllOperations];
    
    /*Attente de la fin des opérations*/
    [motionQueue waitUntilAllOperationsAreFinished];
}

-(void) saveProcessingQueueToFile{
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [paths objectAtIndex:0];
    NSString * datas = @"";
    for (int i =0; i < [frameProcessingQueue count] ; i++) {
        Frame* frame = [frameProcessingQueue frameAtIndex:i];
        for (MotionCapture* capture in [frame motions]) {
            datas = [NSString stringWithFormat:@"%@\n%d\t\t%f\t%f\t%f\t%f\t%f\t%f",
                     datas,
                     frame.frameNumber,
                     capture.attitude.roll, capture.attitude.pitch, capture.attitude.yaw,
                     capture.userAcceleration.x, capture.userAcceleration.y, capture.userAcceleration.z];
        }
    }
    [datas writeToFile:[path stringByAppendingPathComponent:@"datasWithAccelerometer"] atomically:YES encoding:NSUTF8StringEncoding error:nil];
    NSLog(@"%@", datas);
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context{
    if([object class] == [MotionMovementCalculOperation class]){
        if([keyPath isEqualToString:@"isFinished"]){
            /*Notifier le delegate*/
        }
    }
    else{
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

@end
