//
//  CircularProcessingQueue.h
//  MotionDetection
//
//  Created by Kevin Toublanc on 30/12/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

#define CircularProcessingQueueDefaultSize 10
#define CircularProcessingQueueDefaultIndex 0
#define CircularProcessingQueueDefaultContent -1

@protocol CircularProcessingQueueDelegate <NSObject>

@optional
-(void) objectWasErasedAtIndex:(NSUInteger)index;
-(void) objectWasAddedAtIndex:(NSUInteger)index;
-(void) objectWillBeErasedAtIndex:(NSUInteger)index;
-(void) objectWillBeAddedAtIndex:(NSUInteger)index;

@end


@interface CircularProcessingQueue : NSObject{
@public
    NSUInteger currentIndex;
    
@protected
    NSMutableArray * processingQueue;
    
@private
    NSUInteger size;
    
}


@property (weak, nonatomic) NSObject<CircularProcessingQueueDelegate> * delegate;

/**
 **Initialisation de la classe
 **
 **/
-(id) init;

/**
 **Initialisation de la classe avec une taille maximale de queue
 **
 **/
- (id)initWithSize:(NSUInteger)maxSize;

/**
 **Ajoute un objet dans la queue de traitement circulaire
 **Si la taille maximale de la queue est atteinte, l'objet est alors inséré en première position
 **/
-(void) addObject:(id)anObject;

/**
 **Retourne l'objet courant de la queue de traitement
 **Peut retourner nil si aucun objet n'est inséré dans la file
 **/
-(id) currentObject;

/**
 **Retourne l'objet ayant été inséré avant l'objet courant
 **Retourne nil si l'objet courant est le premier objet ou si aucun objet n'a été inséré
 **
 **/
-(id) previousObject;

/**
 **Retourne le n-ième objet ayant été inséré avant l'objet courant
 **Retourne nil si l'objet courant est le premier objet ou si aucun objet n'a été inséré
 **/
-(id) previousObject:(NSUInteger)n;

/**
 **Retourne l'objet à l'index spécifié
 **Si aucun objet n''est disponible alors la fonction retourne nil
 **/
-(id) objectAtIndex:(NSUInteger)index;

/**
 **Retourne le nombre d'objets dans la queue
 **
 **/
-(NSUInteger) count;


@end
