//
//  CircularProcessingQueue.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 30/12/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "CircularProcessingQueue.h"
#import "NSException+Custom.h"
@implementation CircularProcessingQueue

@synthesize delegate;



- (id)init {
    self = [super init];
    if (self) {
        currentIndex = CircularProcessingQueueDefaultIndex;
        size = CircularProcessingQueueDefaultSize;
        processingQueue = [[NSMutableArray alloc] init];
    }
    return self;
}

- (id)initWithSize:(NSUInteger)maxSize{
    self = [super init];
    if (self) {
        currentIndex = CircularProcessingQueueDefaultIndex;
        size = maxSize;
        processingQueue = [[NSMutableArray alloc] init];
    }
    return self;
}

-(void) addObject:(id)anObject{
    @synchronized(self){
        if([processingQueue count] < size){
            
            /*Notification du futur ajout de l'objet*/
            if(delegate && [delegate respondsToSelector:@selector(objectWillBeAddedAtIndex:)]){
                [delegate objectWillBeAddedAtIndex:currentIndex];
            }
            
            [processingQueue addObject:anObject];
            
            /*Notification de l'ajout d'un objet*/
            if(delegate && [delegate respondsToSelector:@selector(objectWasAddedAtIndex:)]){
                [delegate objectWasAddedAtIndex:currentIndex];
            }
        }
        else{
            if(currentIndex >= size){
                currentIndex = CircularProcessingQueueDefaultIndex;
            }
            
            if([processingQueue objectAtIndex:currentIndex]){
                /*Notification de la future suppression de l'objet*/
                if(delegate && [delegate respondsToSelector:@selector(objectWillBeErasedAtIndex:)]){
                    [delegate objectWillBeErasedAtIndex:currentIndex];
                }
                
                /*Suppression de l'objet*/
                [processingQueue removeObjectAtIndex:currentIndex];
                
                /*Notifications si suppression d'un objet*/
                if(delegate && [delegate respondsToSelector:@selector(objectWasErasedAtIndex:)]){
                    [delegate objectWasErasedAtIndex:currentIndex];
                }
            }
            
            /*Notification du futur ajout de l'objet*/
            if(delegate && [delegate respondsToSelector:@selector(objectWillBeAddedAtIndex:)]){
                [delegate objectWillBeAddedAtIndex:currentIndex];
            }
            
            /*Ajout du nouvel objet*/
            [processingQueue insertObject:anObject atIndex:currentIndex];
            
            /*Notification de l'ajout d'un objet*/
            if(delegate && [delegate respondsToSelector:@selector(objectWasAddedAtIndex:)]){
                [delegate objectWasAddedAtIndex:currentIndex];
            }
        }
        currentIndex++;
    }
}


-(id) currentObject{
    @synchronized(self){
        if([processingQueue count] == 0)
            return nil;
        
        if(currentIndex >= size)
            currentIndex = CircularProcessingQueueDefaultIndex;
        
        if([processingQueue count] == size){
            return ((currentIndex == CircularProcessingQueueDefaultIndex)
                    ? [processingQueue lastObject]
                    : [processingQueue objectAtIndex:(currentIndex-1)]);
        }
        else{
            return [processingQueue lastObject];
        }
    }
}

-(id) previousObject{
    @synchronized(self){
        if([processingQueue count] == size){
            /*La queue est remplie*/
            if(currentIndex == 1){
                return [processingQueue lastObject];
            }
            else{
                return [processingQueue objectAtIndex:currentIndex-2];
            }
        }
        else{
            /*La queue n'est pas remplie*/
            if(currentIndex >= 3){
                return [processingQueue objectAtIndex:currentIndex-2];
            }
        }
        return nil;
    }
}

-(id) previousObject:(NSUInteger)n{
    
    @synchronized(self){
        if([processingQueue count] != size){
            /*Si la queue n'est pas remplie*/
            if(n < currentIndex){
                return [processingQueue objectAtIndex:(currentIndex-1-n)];
            }
        }
        else{
            /*La Queue n'est pas remplie*/
            NSInteger objectIndex = currentIndex - 1 - (n % size);
            if(objectIndex >= 0){
                return [processingQueue objectAtIndex:objectIndex];
            }
            else{
                return [processingQueue objectAtIndex:(size + objectIndex)];
            }
        }
    }
    
    return nil;
}

-(id) objectAtIndex:(NSUInteger)index{
    @synchronized(self){
        if([processingQueue count] <= index) @throw [NSException IndexOutOfBoundsException:index];
        return [processingQueue objectAtIndex:index];
    }
}

-(NSUInteger) count{
    @synchronized(self){
        return [processingQueue count];
    }
}

-(BOOL) isfull{
    return [processingQueue count] == size? YES : NO;
}

@end
