//
//  CircularProcessingQueueTests.m
//  MotionDetection
//
//  Created by Kevin Toublanc on 04/01/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CircularProcessingQueueTests.h"

#import <UIKit/UIKit.h>
//#import "application_headers" as required

@implementation CircularProcessingQueueTests

@synthesize processingQueue=_processingQueue;

-(CircularProcessingQueue *)processingQueue{
    if(!_processingQueue){
        _processingQueue = [[CircularProcessingQueue alloc] initWithSize:10];
    }
    return _processingQueue;
}

-(void)setUp{
    for (int i =0; i<=20; i++) {
        [[self processingQueue] addObject:[NSNumber numberWithInt:i]];
    }
}

-(void) testInsert{

    STAssertEquals(20, [((NSNumber*)[[self processingQueue] objectAtIndex:0]) intValue], @"Le premier élément n'est pas cohérent");
    STAssertEquals(20, [((NSNumber*)[[self processingQueue] currentObject]) intValue], @"L'élément courant n'est pas le premier");
}

-(void) testPrevious{
    STAssertEquals(19, [((NSNumber*)[[self processingQueue] previousObject]) intValue], @"L'élément précédent est incohérent");
    for (int i =21; i<=30; i++) {
        [[self processingQueue] addObject:[NSNumber numberWithInt:i]];
        STAssertEquals(i-1, [((NSNumber*)[[self processingQueue] previousObject]) intValue], @"L'élément précédent est incohérent");
    }
}

-(void) testNPreviousWithFullQueue{
    STAssertEquals(20, [((NSNumber*)[[self processingQueue] previousObject:0]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(19, [((NSNumber*)[[self processingQueue] previousObject:1]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(20, [((NSNumber*)[[self processingQueue] previousObject:20]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(11, [((NSNumber*)[[self processingQueue] previousObject:49]) intValue], @"L'élément précédent est incohérent");
    [[self processingQueue] addObject:[NSNumber numberWithInt:21]];
    STAssertEquals(20, [((NSNumber*)[[self processingQueue] previousObject:11]) intValue], @"L'élément précédent est incohérent");

    for (int i =22; i<=35; i++) {
        [[self processingQueue] addObject:[NSNumber numberWithInt:i]];
    }
    STAssertEquals(34, [((NSNumber*)[[self processingQueue] previousObject:11]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(33, [((NSNumber*)[[self processingQueue] previousObject:12]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(32, [((NSNumber*)[[self processingQueue] previousObject:13]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(31, [((NSNumber*)[[self processingQueue] previousObject:14]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(30, [((NSNumber*)[[self processingQueue] previousObject:15]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(29, [((NSNumber*)[[self processingQueue] previousObject:16]) intValue], @"L'élément précédent est incohérent");    
    STAssertEquals(28, [((NSNumber*)[[self processingQueue] previousObject:17]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(27, [((NSNumber*)[[self processingQueue] previousObject:18]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(26, [((NSNumber*)[[self processingQueue] previousObject:19]) intValue], @"L'élément précédent est incohérent");    
    STAssertEquals(35, [((NSNumber*)[[self processingQueue] previousObject:20]) intValue], @"L'élément précédent est incohérent");
}


-(void) testNPreviousWithNotFulledQueue{
    _processingQueue = [[CircularProcessingQueue alloc] initWithSize:10];
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:0], @"La fonction aurait dûe retourner nil");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject], @"La fonction aurait dûe retourner nil");
    for (int i =0; i<=5; i++) {
        [[self processingQueue] addObject:[NSNumber numberWithInt:i]];
    }
    STAssertEquals(4, [((NSNumber*)[[self processingQueue] previousObject:1]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(3, [((NSNumber*)[[self processingQueue] previousObject:2]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(2, [((NSNumber*)[[self processingQueue] previousObject:3]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(1, [((NSNumber*)[[self processingQueue] previousObject:4]) intValue], @"L'élément précédent est incohérent");
    STAssertEquals(0, [((NSNumber*)[[self processingQueue] previousObject:5]) intValue], @"L'élément précédent est incohérent");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:6], @"La fonction aurait dûe retourner nil");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:7], @"La fonction aurait dûe retourner nil");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:8], @"La fonction aurait dûe retourner nil");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:9], @"La fonction aurait dûe retourner nil");
    STAssertEqualObjects(nil, [[self processingQueue] previousObject:10], @"La fonction aurait dûe retourner nil");

}

@end
